@@ -1,27 +1,37 @@
+from functools import lru_cache
+
 import budoux
 from bs4 import BeautifulSoup
 from bs4.element import NavigableString, Tag
 from sphinx.application import Sphinx
+from sphinx.config import Config
 from sphinx.util import logging
 
 logger = logging.getLogger(__name__)
 
 
+def rebuild_contents(elm: Tag, config: Config):
+    parser = lru_cache(budoux.load_default_japanese_parser)()
+    tag = Tag(name=config.budoux_split_tag, can_be_empty_element=True)
+    contents = []
+    for content in elm.contents:
+        if not isinstance(content, NavigableString):
+            contents.append(content)
+            continue
+        tokens = parser.parse(content)
+        contents += sum([[NavigableString(token), tag] for token in tokens], [])
+    return contents
+
+
 def apply_budoux(app: Sphinx, page_name, template_name, context, doctree):
     if "body" not in context:
         return
 
-    def rebuild_title(content: str):
-        parser = budoux.load_default_japanese_parser()
-        tag = Tag(name=app.config.budoux_split_tag, can_be_empty_element=True)
-        return sum([[NavigableString(text), tag] for text in parser.parse(content)], [])
-
     soup = BeautifulSoup(context["body"], "html.parser")
     for tag in app.config.budoux_targets:
         for elm in soup.find_all(tag):
-            new_title = rebuild_title(elm.contents[0])
             elm.attrs["style"] = app.config.budoux_split_style
-            elm.contents = new_title[:-1] + elm.contents[1:]
+            elm.contents = rebuild_contents(elm, app.config)
     context["body"] = soup
     return
 
